﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Sources:
# https://stackoverflow.com/questions/6418678/resize-the-terminal-with-python
# http://tbrink.science/blog/2017/04/30/processing-the-output-of-a-subprocess-with-python-in-realtime/
# https://stackoverflow.com/questions/31926470/run-command-and-get-its-stdout-stderr-separately-in-near-real-time-like-in-a-te

import errno
import os
import pty
import sys
import subprocess
import termios
import struct
import fcntl

def set_winsize(fd, row, col, xpix=0, ypix=0):
    winsize = struct.pack("HHHH", row, col, xpix, ypix)
    fcntl.ioctl(fd, termios.TIOCSWINSZ, winsize)

# parse command in the 1st argument
pty_width = int(sys.argv[1])
pty_height = int(sys.argv[2])
command = sys.argv[3]
args = command.split(" ")
for i, arg in enumerate(args):
    args[i] = args[i].replace("__SPACE__", " ")

# open terminal and set its window size
out_r, out_w = pty.openpty()
set_winsize(out_w, pty_height, pty_width)

# spawn child process and redirect output to PTY
proc = subprocess.Popen(args, bufsize=0, stdout=out_w, stderr=out_w)
os.close(out_w) # if we do not write to process, close this.

while True:
    try:
        output = os.read(out_r, 100000).decode("utf-8")
        sys.stdout.write(output.encode("utf-8"))
    except OSError as e:
        if e.errno != errno.EIO: raise
        output = ""
    if not output:
		sys.exit(proc.wait())